import React from 'react';
import { VariableSizeList } from 'react-window';
import Tree, { createTreeComputer } from './Tree';
import { shouldUpdateRecords, updateRecord, updateRecordOnWalk } from './utils';
const computeTree = createTreeComputer({
  createRecord: (data, {
    recomputeTree,
    resetAfterId
  }) => {
    const record = {
      data,
      height: data.defaultHeight,
      isOpen: data.isOpenByDefault,

      resize(height, shouldForceUpdate) {
        record.height = height;
        resetAfterId(record.data.id, shouldForceUpdate);
      },

      toggle() {
        record.isOpen = !record.isOpen;
        return recomputeTree({
          refreshNodes: record.isOpen,
          useDefaultHeight: true
        });
      }

    };
    return record;
  },
  shouldUpdateRecords: options => {
    var _options$useDefaultHe;

    return shouldUpdateRecords(options) || ((_options$useDefaultHe = options.useDefaultHeight) != null ? _options$useDefaultHe : false);
  },
  updateRecord: (record, recordId, options) => {
    if (options.useDefaultHeight) {
      record.height = record.data.defaultHeight;
    }

    updateRecord(record, recordId, options);
  },
  updateRecordOnWalk: (record, options) => {
    updateRecordOnWalk(record, options);

    if (options.useDefaultHeight) {
      record.height = record.data.defaultHeight;
    }
  }
});
export class VariableSizeTree extends Tree {
  constructor(props, context) {
    super(props, context);
    this.getItemSize = this.getItemSize.bind(this);
    this.state = { ...this.state,
      computeTree,
      resetAfterId: this.resetAfterId.bind(this)
    };
  }

  resetAfterId(id, shouldForceUpdate = false) {
    var _this$list$current;

    (_this$list$current = this.list.current) == null ? void 0 : _this$list$current.resetAfterIndex(this.state.order.indexOf(id), shouldForceUpdate);
  }

  render() {
    const {
      children,
      itemSize,
      rowComponent,
      treeWalker,
      ...rest
    } = this.props;
    return /*#__PURE__*/React.createElement(VariableSizeList, Object.assign({}, rest, {
      itemData: this.state,
      itemCount: this.state.order.length // eslint-disable-next-line @typescript-eslint/unbound-method
      ,
      itemSize: itemSize != null ? itemSize : this.getItemSize,
      ref: this.list
    }), rowComponent);
  }

  getItemSize(index) {
    const {
      order,
      records
    } = this.state;
    return records[order[index]].height;
  }

}